@@ -10,6 +10,7 @@ from django.core.files.base import ContentFile
 from django.core.validators import (
     BaseValidator,
     DecimalValidator,
+    DomainNameValidator,
     EmailValidator,
     FileExtensionValidator,
     MaxLengthValidator,
@@ -22,6 +23,7 @@ from django.core.validators import (
     URLValidator,
     int_list_validator,
     validate_comma_separated_integer_list,
+    validate_domain_name,
     validate_email,
     validate_image_file_extension,
     validate_integer,
@@ -465,11 +467,39 @@ TEST_DATA = [
     (StepValueValidator(3), 1, ValidationError),
     (StepValueValidator(3), 8, ValidationError),
     (StepValueValidator(3), 9, None),
+    (StepValueValidator(2), 4, None),
+    (StepValueValidator(2, offset=1), 3, None),
+    (StepValueValidator(2, offset=1), 4, ValidationError),
     (StepValueValidator(0.001), 0.55, None),
     (StepValueValidator(0.001), 0.5555, ValidationError),
+    (StepValueValidator(0.001, offset=0.0005), 0.5555, None),
+    (StepValueValidator(0.001, offset=0.0005), 0.555, ValidationError),
     (StepValueValidator(Decimal(0.02)), 0.88, None),
     (StepValueValidator(Decimal(0.02)), Decimal(0.88), None),
     (StepValueValidator(Decimal(0.02)), Decimal(0.77), ValidationError),
+    (StepValueValidator(Decimal(0.02), offset=Decimal(0.01)), Decimal(0.77), None),
+    (StepValueValidator(Decimal(2.0), offset=Decimal(0.1)), Decimal(0.1), None),
+    (
+        StepValueValidator(Decimal(0.02), offset=Decimal(0.01)),
+        Decimal(0.88),
+        ValidationError,
+    ),
+    (StepValueValidator(Decimal("1.2"), offset=Decimal("2.2")), Decimal("3.4"), None),
+    (
+        StepValueValidator(Decimal("1.2"), offset=Decimal("2.2")),
+        Decimal("1.2"),
+        ValidationError,
+    ),
+    (
+        StepValueValidator(Decimal("-1.2"), offset=Decimal("2.2")),
+        Decimal("1.1"),
+        ValidationError,
+    ),
+    (
+        StepValueValidator(Decimal("-1.2"), offset=Decimal("2.2")),
+        Decimal("1.0"),
+        None,
+    ),
     (URLValidator(EXTENDED_SCHEMES), "file://localhost/path", None),
     (URLValidator(EXTENDED_SCHEMES), "git://example.com/", None),
     (
@@ -601,6 +631,48 @@ TEST_DATA = [
     (ProhibitNullCharactersValidator(), "\x00something", ValidationError),
     (ProhibitNullCharactersValidator(), "something", None),
     (ProhibitNullCharactersValidator(), None, None),
+    (validate_domain_name, "000000.org", None),
+    (validate_domain_name, "python.org", None),
+    (validate_domain_name, "python.co.uk", None),
+    (validate_domain_name, "python.tk", None),
+    (validate_domain_name, "domain.with.idn.tld.उदाहरण.परीक्ष", None),
+    (validate_domain_name, "ıçğü.com", None),
+    (validate_domain_name, "xn--7ca6byfyc.com", None),
+    (validate_domain_name, "hg.python.org", None),
+    (validate_domain_name, "python.xyz", None),
+    (validate_domain_name, "djangoproject.com", None),
+    (validate_domain_name, "DJANGOPROJECT.COM", None),
+    (validate_domain_name, "spam.eggs", None),
+    (validate_domain_name, "python-python.com", None),
+    (validate_domain_name, "python.name.uk", None),
+    (validate_domain_name, "python.tips", None),
+    (validate_domain_name, "例子.测试", None),
+    (validate_domain_name, "dashinpunytld.xn---c", None),
+    (validate_domain_name, "python..org", ValidationError),
+    (validate_domain_name, "python-.org", ValidationError),
+    (validate_domain_name, "too-long-name." * 20 + "com", ValidationError),
+    (validate_domain_name, "stupid-name试", ValidationError),
+    (validate_domain_name, "255.0.0.0", ValidationError),
+    (validate_domain_name, "fe80::1", ValidationError),
+    (validate_domain_name, "1:2:3:4:5:6:7:8", ValidationError),
+    (DomainNameValidator(accept_idna=False), "non-idna-domain-name-passes.com", None),
+    (
+        DomainNameValidator(accept_idna=False),
+        "domain.with.idn.tld.उदाहरण.परीक्ष",
+        ValidationError,
+    ),
+    (DomainNameValidator(accept_idna=False), "ıçğü.com", ValidationError),
+    (DomainNameValidator(accept_idna=False), "not-domain-name", ValidationError),
+    (
+        DomainNameValidator(accept_idna=False),
+        "not-domain-name, but-has-domain-name-suffix.com",
+        ValidationError,
+    ),
+    (
+        DomainNameValidator(accept_idna=False),
+        "not-domain-name.com, but has domain prefix",
+        ValidationError,
+    ),
 ]
 
 # Add valid and invalid URL tests.
@@ -673,24 +745,24 @@ class TestValidatorEquality(TestCase):
 
     def test_regex_equality(self):
         self.assertEqual(
-            RegexValidator(r"^(?:[a-z0-9\.\-]*)://"),
-            RegexValidator(r"^(?:[a-z0-9\.\-]*)://"),
+            RegexValidator(r"^(?:[a-z0-9.-]*)://"),
+            RegexValidator(r"^(?:[a-z0-9.-]*)://"),
         )
         self.assertNotEqual(
-            RegexValidator(r"^(?:[a-z0-9\.\-]*)://"),
-            RegexValidator(r"^(?:[0-9\.\-]*)://"),
+            RegexValidator(r"^(?:[a-z0-9.-]*)://"),
+            RegexValidator(r"^(?:[0-9.-]*)://"),
         )
         self.assertEqual(
-            RegexValidator(r"^(?:[a-z0-9\.\-]*)://", "oh noes", "invalid"),
-            RegexValidator(r"^(?:[a-z0-9\.\-]*)://", "oh noes", "invalid"),
+            RegexValidator(r"^(?:[a-z0-9.-]*)://", "oh noes", "invalid"),
+            RegexValidator(r"^(?:[a-z0-9.-]*)://", "oh noes", "invalid"),
         )
         self.assertNotEqual(
-            RegexValidator(r"^(?:[a-z0-9\.\-]*)://", "oh", "invalid"),
-            RegexValidator(r"^(?:[a-z0-9\.\-]*)://", "oh noes", "invalid"),
+            RegexValidator(r"^(?:[a-z0-9.-]*)://", "oh", "invalid"),
+            RegexValidator(r"^(?:[a-z0-9.-]*)://", "oh noes", "invalid"),
         )
         self.assertNotEqual(
-            RegexValidator(r"^(?:[a-z0-9\.\-]*)://", "oh noes", "invalid"),
-            RegexValidator(r"^(?:[a-z0-9\.\-]*)://"),
+            RegexValidator(r"^(?:[a-z0-9.-]*)://", "oh noes", "invalid"),
+            RegexValidator(r"^(?:[a-z0-9.-]*)://"),
         )
 
         self.assertNotEqual(
@@ -704,7 +776,7 @@ class TestValidatorEquality(TestCase):
         )
 
     def test_regex_equality_nocache(self):
-        pattern = r"^(?:[a-z0-9\.\-]*)://"
+        pattern = r"^(?:[a-z0-9.-]*)://"
         left = RegexValidator(pattern)
         re.purge()
         right = RegexValidator(pattern)
@@ -733,6 +805,10 @@ class TestValidatorEquality(TestCase):
             EmailValidator(message="BAD EMAIL", code="bad"),
             EmailValidator(message="BAD EMAIL", code="bad"),
         )
+        self.assertEqual(
+            EmailValidator(allowlist=["127.0.0.1", "localhost"]),
+            EmailValidator(allowlist=["localhost", "127.0.0.1"]),
+        )
 
     def test_basic_equality(self):
         self.assertEqual(
@@ -787,6 +863,10 @@ class TestValidatorEquality(TestCase):
             FileExtensionValidator(["TXT", "png"]),
             FileExtensionValidator(["txt", "png"]),
         )
+        self.assertEqual(
+            FileExtensionValidator(["jpg", "png", "txt"]),
+            FileExtensionValidator(["txt", "jpg", "png"]),
+        )
         self.assertEqual(
             FileExtensionValidator(["txt"]),
             FileExtensionValidator(["txt"], code="invalid_extension"),
@@ -822,3 +902,25 @@ class TestValidatorEquality(TestCase):
             ProhibitNullCharactersValidator(message="message", code="code1"),
             ProhibitNullCharactersValidator(message="message", code="code2"),
         )
+
+    def test_domain_name_equality(self):
+        self.assertEqual(
+            DomainNameValidator(),
+            DomainNameValidator(),
+        )
+        self.assertNotEqual(
+            DomainNameValidator(),
+            EmailValidator(),
+        )
+        self.assertNotEqual(
+            DomainNameValidator(),
+            DomainNameValidator(code="custom_code"),
+        )
+        self.assertEqual(
+            DomainNameValidator(message="custom error message"),
+            DomainNameValidator(message="custom error message"),
+        )
+        self.assertNotEqual(
+            DomainNameValidator(message="custom error message"),
+            DomainNameValidator(message="custom error message", code="custom_code"),
+        )
